%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% For paper: Dynamics of particle aggregation in de-wetting films of complex liquids
% Junzhe (James) Zhang, David Sibley, Dmitri Tseluiko, Andrew Archer
% Loughborough University
% 11/2023
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

clear;
close all;
% All simulations use random seeds 11 %
rand('state',11);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Set up for Figure 5 %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% % Note: This takes a long time to compute and consumes significant RAM
 
% tplot = 100; % Number of frames plotted
% tend = 10^12; % Final time (unscaled)
% Lx = 200; %System length
% 
% % Node and mesh size
% n = 1000; % Length of array (h1, h2 ...h end, psi 1... psi_end; actual step number n/2)
% dx = Lx/(n/2); % Step size
% x = linspace(0,Lx-dx,n/2); % Number of points in X-direction
% 
% 
% %Set parameter
% frac = 1; % Strength of coupling
% alpha = 1 * frac; % This is alpha dash, controls attraction
% beta = alpha; % This is beta dash, controls repulsion (Note the alpha/beta ratio is 1)
% gamma = 1; % Surface tension coef
% eta0 = 1/3; % Fixed mobility 
% evap = 0; % No evaporation considered
% 
% kTa3 = 0.15 * frac; % (k_B*T)/a^3, temperature term
% epsilon = 0.5 * frac; % Interfacial tension
% phi_i = 0.4; % Initial concentration
% A = 2; % This is A', coefficient in binding potential
% B = A; % This is B', coefficient in binding potential
% a02 = 2; % This is relative particle size a^2
% h_i = 2.2; % Initial film height
% psi_i = h_i * phi_i; %Initial effective height

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Set up for Figure 7 %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% tplot = 100; % Number of frames plotted
% tend = 200000; % Final time (unscaled)
% Lx = 300; %System length
% 
% % Node and mesh size
% n = 1000; % Length of array (h1, h2 ...h end, psi 1... psi_end; actual step number n/2)
% dx = Lx/(n/2); % Step size
% x = linspace(0,Lx-dx,n/2); % Number of points in X-direction
% 
% %Set parameter
% frac = 100; % Strength of coupling
% alpha = 1 * frac; % This is alpha dash, controls attraction
% beta = alpha; % This is beta dash, controls repulsion (Note the alpha/beta ratio is 1)
% gamma = 1; % Surface tension coef
% eta0 = 1/3; % Fixed mobility 
% evap = 0; % No evaporation considered

%%%%%% (a) (b) (c) %%%%%%
 
% kTa3 = 0.11 * frac; % (k_B*T)/a^3, temperature term
% epsilon = 40 * frac; % Interfacial tension
% phi_i = 0.4; % Initial concentration
% A = 1; % This is A', coefficient in binding potential
% B = A; % This is B', coefficient in binding potential
% a02 = 100; % This is relative particle size a^2
% h_i = 2.5; % Initial film height
% psi_i = h_i * phi_i; %Initial effective height

%%%%%% (d) (e) (f) %%%%%%

% kTa3 = 0.11 * frac; % (k_B*T)/a^3, temperature term
% epsilon = 40 * frac; % Interfacial tension
% phi_i = 0.3; % Initial concentration
% A = 1; % This is A', coefficient in binding potential
% B = A; % This is B', coefficient in binding potential
% a02 = 100; % This is relative particle size a^2
% h_i = 2.5; % Initial film height
% psi_i = h_i * phi_i; %Initial effective height

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Set up for Figure 8 %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% tplot = 100; % Number of frames plotted
% tend = 10^12; % Final time (unscaled)
% Lx = 200; %System length
% 
% % Node and mesh size
% n = 1000; % Length of array (h1, h2 ...h end, psi 1... psi_end; actual step number n/2)
% dx = Lx/(n/2); % Step size
% x = linspace(0,Lx-dx,n/2); % Number of points in X-direction 
% 
% %Set parameter
% frac = 1; % Strength of coupling
% alpha = 1 * frac; % This is alpha dash, controls attraction
% beta = alpha; % This is beta dash, controls repulsion (Note the alpha/beta ratio is 1)
% gamma = 1; % Surface tension coef
% eta0 = 1/3; % Fixed mobility 
% evap = 0; % No evaporation considered
 
%%%%%% (a) %%%%%%
 
% kTa3 = 0.15 * frac; % (k_B*T)/a^3, temperature term
% epsilon = 0.4 * frac; % Interfacial tension
% phi_i = 0.15; % Initial concentration
% A = 1; % This is A', coefficient in binding potential
% B = A; % This is B', coefficient in binding potential
% a02 = 100; % This is relative particle size a^2
% h_i = 2.2; % Initial film height
% psi_i = h_i * phi_i; %Initial effective height
 
%%%%%% (b) %%%%%%
 
% kTa3 = 0.15 * frac; % (k_B*T)/a^3, temperature term
% epsilon = 0.4 * frac; % Interfacial tension
% phi_i = 0.17; % Initial concentration
% A = 1; % This is A', coefficient in binding potential
% B = A; % This is B', coefficient in binding potential
% a02 = 100; % This is relative particle size a^2
% h_i = 1.9; % Initial film height
% psi_i = h_i * phi_i; %Initial effective height
% 
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Set up for Figure 9 %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

tplot = 250; % Number of frames plotted
tend = 10^6; % Final time (unscaled)
Lx = 60; %System length

% Node and mesh size
n = 1000; % Length of array (h1, h2 ...h end, psi 1... psi_end; actual step number n/2)
dx = Lx/(n/2); % Step size
x = linspace(0,Lx-dx,n/2); % Number of points in X-direction


%Set parameter
frac = 3; % Strength of coupling
alpha = 1 * frac; % This is alpha dash, controls attraction
beta = alpha; % This is beta dash, controls repulsion (Note the alpha/beta ratio is 1)
gamma = 1; % Surface tension coef
eta0 = 1/3; % Fixed mobility 
evap = 0; % No evaporation considered

kTa3 = 0.13; % (k_B*T)/a^3, temperature term
epsilon = 0.5; % Interfacial tension
phi_i = 0.41; % Initial concentration
A = 1; % This is A', coefficient in biding potential
B = A; % This is B', coefficient in biding potential
a02 = 2; % This is relative particle size a^2
h_i = 2.5; % Initial film height
psi_i = h_i * phi_i; %Initial effective height


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Matrix operators function %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

[DX,DX2] = fdder1dcyc(n,dx);
AA = DX;
BB = DX2;


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Initial condition %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% % For Figure 5, 7, 8
% % h0 = h_i + 0.0000001 * (rand(n/2,1) - 0.5);
% For Figure 9
h0 = h_i + 0.00001 * (rand(n/2,1) - 0.5);
h = h0;
% initial effective height 
psi0 = psi_i + 0.00001 * (rand(n/2,1) - 0.5);
hp0 = [h0' psi0'];


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Set up for time steps and ode solvers %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% For Figure 7 and 8
tspan = linspace(10^(-6),tend,tplot);
% For Figure 5 and 9
tspanplot = logspace(-6,log10(tend),tplot);
options = odeset('RelTol',1e-9,'AbsTol',1e-9);

sol = ode15s(@(t,hp)dHedt(t,hp,AA,BB,gamma,evap,kTa3,alpha,beta,epsilon,A,B,eta0,a02), tspan, hp0, options); 
% % For Figure 7 and 8
% ht = deval(sol,tspan);
% For Figure 5 and 9
ht = deval(sol,tspanplot);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Setting up variables in graphs %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

hpp   = ht(1:end/2,:);
psipp = ht(end/2+1:end,:);%hpp and psipp are for animation plotting
phipp = psipp./hpp;
xp    = x;
up    = transpose(ht);
hup   = up(:,1:end/2);
psiup = up(:,end/2+1:end);%hup and psiup are for final result plotting


%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Plotting %%%%%%
%%%%%%%%%%%%%%%%%%%%%%

% This figure is for the final result.
figure(1); hold on
plot(xp,hup(end,:),'-b','LineWidth',1); % Plot for 
plot(xp,psiup(end,:),'--r','LineWidth',1);
plot(xp,psiup(end,:)./hup(end,:),'-.m','LineWidth',1.5);
%%%% Figure 7 %%%%
% yline(0.08,'--','$\phi_1$','Interpreter','LaTex','FontSize',23);
% yline(0.6,'--','$\phi_2$','Interpreter','LaTex','FontSize',23);
%%%% Figure 9 %%%%
yline(0.00056,'--','$\phi_1$','Interpreter','LaTex','FontSize',23);
yline(0.77,'--','$\phi_2$','Interpreter','LaTex','FontSize',23);
set(gca,'FontSize',20)
xlim([0 Lx]);
ylim([0 4]);
dim3 = [0.13 0.55 0.3 0.3];
str3 = {'(c)'};
annotation('textbox',dim3,'String',str3,'FitBoxToText','on','Interpreter','latex','FontSize',30,'EdgeColor','none');

xlabel('$x$','Interpreter','LaTex','FontSize',25);
ylabel('$z\qquad$','Interpreter','LaTex','FontSize',25,'Rotation',0);
legend('$h$','$\psi$','$\phi$','Interpreter','LaTex','FontSize',25,'Location','northeast');
box on
hold off

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%% Figure 5, 7, 8 waterfall plot %%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% figure(2);
% sh = waterfall(xp,tspanplot,hpp'); colormap(1e-6*[1 1 1]); view(-15,80) %view(-10,80) %view(-20,25)
% sh.EdgeColor = 'k';
% axis([0 Lx tspanplot(1) tspanplot(end) (floor(min(min(hpp))*100)-1)/100 (floor(max(max(hpp))*100)+1)/100]);
% grid off
% set(gca,'ztick',[(floor(min(min(hpp))*100)-1)/100 (floor(max(max(hpp))*100)+1)/100]), 
% ylim([0 tend])
% pbaspect([2 1.5 1.5])
% set(gca,'FontSize',20)
% dim2 = [0.17 0.55 0.3 0.3];
% str2 = {'(d)'};
% annotation('textbox',dim2,'String',str2,'FitBoxToText','on','Interpreter','latex','FontSize',30,'EdgeColor','none');
% xlabel('$x$','Interpreter','latex','FontSize',30);
% ylabel('$t$','Interpreter','latex','FontSize',30,'Rotation',0); 
% zlabel('$h$','Interpreter','latex','FontSize',30,'Rotation',0); 

% figure(3);
% sh = waterfall(xp,tspanplot,phipp'); colormap(1e-6*[1 1 1]); view(-15,80) %view(-10,80) %view(-20,25)
% sh.EdgeColor = 'k';
% axis([0 Lx tspanplot(1) tspanplot(end) (floor(min(min(phipp))*100)-1)/100 (floor(max(max(phipp))*100)+1)/100]);
% grid off
% set(gca,'ztick',[(floor(min(min(phipp))*100)-1)/100 (floor(max(max(phipp))*100)+1)/100]) 
% ylim([10^3 tend])
% pbaspect([2 1.5 1.5])
% set(gca,'FontSize',20)
% dim3 = [0.13 0.55 0.3 0.3];
% str3 = {'(b)'};
% annotation('textbox',dim3,'String',str3,'FitBoxToText','on','Interpreter','latex','FontSize',30,'EdgeColor','none');
% xlabel('$x$','Interpreter','latex','FontSize',30);
% ylabel('$t$','Interpreter','latex','FontSize',30,'Rotation',0); 
% zlabel('$\phi$','Interpreter','latex','FontSize',30,'Rotation',0); 

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Figure 9 waterfall %%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

figure(2);
%set(fh,'defaulttextinterpreter','latex');
sh = waterfall(xp,tspanplot,hpp'); colormap(1e-6*[1 1 1]); view(-15,80) %view(-10,80) %view(-20,25)
sh.EdgeColor = 'k';
yticks([10^2 10^3 10^4 10^5]) 
axis([0 Lx tspanplot(1) tspanplot(end) (floor(min(min(hpp))*100)-1)/100 (floor(max(max(hpp))*100)+1)/100]);
grid off
ylim([50 10^5])
set(gca,'ztick',[(floor(min(min(hpp))*100)-1)/100 (floor(max(max(hpp))*100)+1)/100]), 
set(gca,'YScale','log')
pbaspect([2 1.5 1.5])
set(gca,'FontSize',20)
xlabel('$x$','Interpreter','latex','FontSize',30);
ylabel('$t$','Interpreter','latex','FontSize',30,'Rotation',0); 
zlabel('$h$','Interpreter','latex','FontSize',30,'Rotation',0);
dim2 = [0.13 0.6 0.3 0.3];
str2 = {'(a)'};
annotation('textbox',dim2,'String',str2,'FitBoxToText','on','Interpreter','latex','FontSize',30,'EdgeColor','none');

figure(3);
sh = waterfall(xp,tspanplot,phipp'); colormap(1e-6*[1 1 1]); view(-15,80) %view(-10,80) %view(-20,25)
sh.EdgeColor = 'k';
yticks([10^2 10^3 10^4 10^5])
zticks([0 0.78]) 
axis([0 Lx tspanplot(1) tspanplot(end) (floor(min(min(phipp))*100)-1)/100 (floor(max(max(phipp))*100)+1)/100]);
grid off
ylim([50 10^5])
set(gca,'ztick',[(floor(min(min(phipp))*100)-1)/100 (floor(max(max(phipp))*100)+1)/100]) 
set(gca,'YScale','log')
pbaspect([2 1.5 1.5])
set(gca,'FontSize',20)
xlabel('$x$','Interpreter','latex','FontSize',30);
ylabel('$t$','Interpreter','latex','FontSize',30,'Rotation',0); 
zlabel('$\phi$','Interpreter','latex','FontSize',30,'Rotation',0); 
dim3 = [0.13 0.6 0.3 0.3];
str3 = {'(b)'};
annotation('textbox',dim3,'String',str3,'FitBoxToText','on','Interpreter','latex','FontSize',30,'EdgeColor','none');


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Plotting for Figure 9(d) %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Note: please use figure 9 parameters

figure(4);% This plots the theoretical dispersion relation
k = linspace(0,10,2001);
p = kTa3 - phi_i + 3*phi_i.^3;
omega_f = -h_i^3*(k.^4+A*(-6/h_i^4+12/h_i^5)*k.^2);
omega_c = -(a02*alpha/(2*pi))*k.^2.*(kTa3/alpha-phi_i+3*beta/alpha*phi_i^3+epsilon/alpha*phi_i*k.^2);
plot(k,omega_f,'b','LineWidth',2);
hold on
plot(k,omega_c,'r--','LineWidth',2);
hold off
grid on
xlim([0 2]);
ylim([-0.15 0.15]);
set(gca,'FontSize',20)
xlabel('\it k','Interpreter','LaTex','FontSize',30);
label_h = ylabel("$\omega$",'Interpreter','latex','FontSize',30,'Rotation',0);
legend('$\omega_h$','$\omega_\psi$','Interpreter','latex','FontSize',25);
dim2 = [0.13 0.62 0.3 0.3];
str2 = {'(d)'};
annotation('textbox',dim2,'String',str2,'FitBoxToText','on','Interpreter','latex','FontSize',30,'EdgeColor','none');
  

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% End of main code %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Function for equation in ode solver %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function TFCSEqn = dHedt(t,hp,AA,BB,gamma,evap,kTa3,alpha,beta,epsilon,A,B,eta0,a02)

h = hp(1:end/2);%film height vector
psi = hp(end/2+1:end);%effective height vector
phi = psi./h;%concentration

eta_r = 1/(3*eta0);

%Phase diagram parameters
h_x = AA * h;
psi_x = AA * psi;
B_psi = BB * psi;

%p = deltaF/deltah
p = -(gamma * BB * h) + ((2 * A)./h.^3 - (3 * B)./h.^4) - kTa3 * phi + (alpha/2) * (phi).^2 -...
    (3 * beta/4) * (phi).^4 + epsilon./h.^2 .* ((-2 * phi .* psi_x .* h_x) +...
    (3/2 * phi.^2 .* h_x.^2) + (1/2 .* psi_x.^2) + (psi .* B_psi) -...
    (psi .* phi .* (BB * h)));
%q = deltaF/deltapsi
q = kTa3 * (log(phi) + 1) - alpha * phi + beta * phi .^3 + epsilon./h.^2 .*...
    ((psi .* (BB * h)) - (phi .* h_x.^2) - (h .* B_psi) + (h_x .* psi_x));

A_p = AA * p;
A_q = AA * q;

% Governing equation
dhdt = AA * (eta_r .* (h.^3) .* A_p + eta_r .* (h.^2 .* psi) .* A_q)-evap*(p+0.001);
dpsidt = AA * (eta_r .* ((psi .* h.^2) .* A_p) + eta_r .* ((psi.^2 .* h + ((a02*psi)/(2*pi))) .* A_q));

TFCSEqn = [dhdt' dpsidt']';

end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%% Function for matrix operators %%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


function [DX,DX2] = fdder1dcyc(n,dx)
% FDDER1D One-Dimensional Finite-Difference Derivative Operators
% Using Dirichlet Boundary Conditions
%
% [DX,DX2] = fdder1d(Nx,dx);
%
% n is the size the grid.
%
% dx is the grid resolution.
%
% DX is a square matrix of size Nx-by-Nx that calculates a first-order
% derivative.
%
% DX2 is a square matrix of size Nx-by-Nx that calculates a second-order
% derivative.

M = n/2;
Z = sparse(M,M);
d = ones(M,1);
B = spdiags(-2*d,0,Z);
B = spdiags(d,-1,B);
B = spdiags(d,1,B);
B = spdiags(d,M-1,B);
B = spdiags(d,-M+1,B);
DX2 = (1/dx^2) * B;

M = n/2;
Z = sparse(M,M);
d = ones(M,1);
A = spdiags(0*d,0,Z);
A = spdiags(-d,-1,A);
A = spdiags(d,1,A);
A = spdiags(-d,M-1,A);
A = spdiags(d,-M+1,A);
DX = (1/(2*dx)) * A;
end

